pro trace_special_movie3, t0, t1, minframes=minframes, maxframes=maxframes, $
                      percentd=percentd, last_nhours=last_nhours, $
		      prepit=prepit,  outsize=outsize,  _extra=_extra, $
		      mdir=mdir, clean=clean, grid=grid, testing=testing, $
		      queue_days=queue_days, context=context, $
		      movie_dir=movie_dir, special_name=special_name, $
		      movie_name=movie_name, noprep=noprep, $
		      interactive=interactive,  exclude=exclude, $
		      minnx=minnx, minny=minny, median_delta=median_delta, $
                      fdroll=fdroll, fd_type=fd_type, normalize=normalize, $
                      radiation_belt=radiation_belt, cube_unspike=cube_unspike, $
                      goes_movie=goes_movie
;+
;   Name: trace_special_movie
;
;   Purpose: TRACE movies -> WWW  
;
;   Keyword Parameters:
;      minframes - ignore movies with fewer than this many frames
;      maxframes - upper limit number of frames per movie
;      outsize   - output size for each movie frame
;      prepit  - switch, if set, run data through trace_prep
;      _EXTRA  - special_movie / image2movie keywords via inheritance
;      grid    - if set, than grid MAXFRAMES over interval
;                (default is last MAXFRAMES images)
;      last_nhours - most recent N hours to consider (default=24)
;      percentd - frame quality threshold (default=100%) 
;      queue_days - number of days saved (passed to trace_last_movie_queue)
;      mdir  - output directory (movies, gifs, etc)
;      movie_dir - synonym for mdir
;      noprep - if set, inhibit trace_prep
;      interactive - if set, sequence selection is interactive
;      context - include EIT/GOES context
;                if 1 element array - size of full size context image
;                if 2 element array - size of [thumbnail,fullsize]
;                if switch (=1) - default [thumb,full] = [ 140,512]
;      cube_unspike - if set, call 'ssw_unspike_cube' (EUV only)
;      goes_movie - if set, append GOES XRay movie plot
;
;   History:
;      25-April-1998 - S.L.Freeland - apply some SSW movie techniques->TRACE  
;      12-Jun-1998   - S.L.Freeland - call 'trace_data_filter' to remove
;                      radiation contaminated/other bad data  
;      17-Jul-1998   - S.L.Freeland - filter out movies < 512x512 (dosimiter)
;      16-Sep-1998   - Add /GRID, change default outsize to 320 (mpeg compat)
;                      Add LAST_NHOURS and PERCENTD keywords and function
;      22-Sep-1998   - keep genx log of movie frame info, add /TESTING keyword
;      27-Sep-1998   - pass MDIR to trace_movie_index
;      13-Oct-1998   - derive from 'trace_last_movie' - permit user time range
;                      enable context images
;      26-May-1999   - make 'prep' and 'derotate' defaults, additional mods
;                      unify (single point maint) recent and 'special' movies
;       1-Jun-1999   - S.L.Freeland - make recent movies in 'offline'
;                      subdirectory - only transfer->default WWW when complete
;      15-Jun-1999   - S.L.Freeland - add /INTERACTIVE, MINNX, EXCLUDE
;                      ( pass through to 'trace_uniq_movies' )  
;      8-Dec-1999    - add FDROLL ( -> ssw_fov_context during soho roll times)
;      8-May-2000    - permit CONTEXT specification of context image size
;     18-Jun-2000    - S.L.Freeland add FD_TYPE ( -> ssw_fov_context )   
;     26-Jun-2000    - add /NORMALIZE to trace_prep call.
;
;   Method:
;      use trace_cat - loop through 'special_movie' for each
;                      uniq movie dset - organize submovies by OBS_PROG
;
;   Calls:
;      trace_cat, trace_cat2data, read_trace, trace_uniq_movies 
;      special_movie/image2movie, set_logenv, get_logenv
;      grid_data, last_nelem
;      html_doc, file_append, the usual suspects
;      ssw_track_fov, ssw_fov_context
;-
normalize=keyword_set(normalize)

if not keyword_set(minx) then minnx = 512

if data_chk(movie_name,/string) and n_elements(special_name) eq 0 then $
    special_name=movie_name
testing=keyword_set(testing)
if data_chk(movie_dir,/string,/scalar) then mdir=movie_dir

; ============ should use configure_http,/trace ============
pathhttp=get_logenv('path_http')
if pathhttp eq '' then pathhttp='/DataCenter/www/htdocs/TRACE/  ; lmsal default
recent=keyword_set(last_nhours)

case 1 of 
   n_elements(mdir) eq 0 and recent:  mxdir=concat_dir(pathhttp,'last_movies_temp')
   keyword_set(mdir) and 1-file_exist(mdir): mxdir=concat_dir(pathhttp,mdir)
   file_exist(mdir): mxdir=mdir
   n_params() gt 0:  mxdir=concat_dir(pathhttp,'special_movies')
   file_exist(concat_dir('$path_http',mdir)): mxdir=concat_dir('$path_http',mdir)
   else: begin
      box_message,'Please define MOVIE_DIR and try again'
      return
   endcase
endcase

if not file_exist(mxdir) then begin
   box_message,'Movie directory does not exist - creating it...'
   spawn,'mkdir -p ' + mxdir
   if not file_exist(mxdir) then begin
      box_message,'Could not create movie dierctory: ' + mxdir + ' , returning...'
      return
  endif
endif

box_message,'Using movie directory: ' + mxdir(0)

case 1 of 
  recent: begin
     top_name='index.html'
     genlog='last_movie_log.genx'
  endcase
  data_chk(special_name,/string): begin
    top_name=special_name(0)+'.html'
    genlog=special_name(0)+'.genx'
  endcase
  else: begin
     top_name='special_movie_'+time2file(t0)+'.html'
    genlog=top_name+'.genx'
  endcase
endcase
genlog=concat_dir(mxdir,genlog)
mname=str_replace(top_name,'.html','')

set_logenv,'path_http',pathhttp

tophttp=get_logenv('top_http')
if tophttp eq '' then tophttp='http://vestige.lmsal.com/TRACE/'
set_logenv,'top_http',tophttp 

if not file_exist(mxdir) then begin
   box_message,['Movie Directory: ' + mxdir +' does not exist',$
		'Create it or use another']
   return
endif  

cd,mxdir

set_plot,'Z'                                            ; use the Z buffer
; ============================================================

prepit=1-keyword_set(noprepit)                        ; trace_prep is default

; --------- read the recent TRACE catalog -----------------
if not keyword_set(last_nhours) and n_params() eq 0 then last_nhours=24
case n_params() of
   0: begin
        nhours=strtrim(last_nhours,2)
        execmd='trace_cat,catalog,/last' + nhours
   endcase
   2: execmd='trace_cat,t0,t1,catalog               ; time range passed
   else: begin
     box_message,'Need start/stop time range OR LAST_NHOURS input'
     return
   endcase
endcase

exestat=execute(execmd)

if not data_chk(catalog,/struct) or n_elements(catalog) le 1 then begin
    if n_elements(nhours) eq 0 then nhours=-1
    box_message,['Last catalog entry >'+nhours+ 'hours old','Aborting...']
    return
endif

; --------- init some defaults ------------------
if not keyword_set(percentd) then percentd=100     ; data quality cutoff
if not keyword_set(outsize) then outsize=320       ; output frame size
origsize=outsize
if n_elements(minframes) eq 0 then minframes=10    ; not enough, dont bother 
if n_elements(maxframes) eq 0 then maxframes=30    ; upper limit
if n_elements(median_delta) eq 0 then median_delta=.2
; -------------------------------------------------
   
; ------ find 'uniq' movies within these catalog entries ---------
defexclude=str2arr('dark,????,testsnap')
if data_chk(exclude,/string) then $
    defexclude=[defexclude,str2arr(exclude)]

umovies=trace_uniq_movies(catalog,minframes=minframes, $  ; uniq movies
			  exclude=defexclude, nosht_mdur=normalize, $
			  minnx=minnx, interactive=interactive)   ; only 'interesting'
; -------------------------------------------------

uobs=uniq(umovies.obs_prog, sort(umovies.obs_prog))  ; number of top level WWW docs
; -------------------------------------------------
logstruct=add_tag(umovies,replicate(catalog(0),maxframes),'CATALOG')
logstruct=add_tag(logstruct,0,'NSS')
logstruct=add_tag(logstruct,0,'DQREJECT')

; --------- for each observing program ------------
allhtml=''                                      ; accumulator for output html
for i=0,n_elements(uobs)-1 do begin             ; for each observing program
   uss=where(umovies.obs_prog eq umovies(uobs(i)).obs_prog,mcnt)
   uss=reverse(uss)
   thisobsprog=umovies(uobs(i)).obs_prog
   box_message,'Number of records for OBS_PROG ' + thisobsprog +' :'+strtrim(mcnt,2) 
   obsheader='<h2>'+ thisobsprog +'</h2>'
   case 1 of
      recent: allhtml=obsheader                ; initialize html header
      else:   allhtml=[allhtml,obsheader]      ; special? append everything
   endcase      

   ;  -------- for each submovie -------------
   for m=0,mcnt-1 do begin 

   ;   -------- map this movie -> SS relative to catalog ---------
      thismovie=umovies(uss(m))                          ; pointer
      box_message,'Processing Sequence >> ' + thismovie.uname, nbox=2
      ss=trace_uniq_movies(catalog,thismovie,nosht_mdur=normalize)       ; subscripts for 1 mov
      ssgood=trace_data_filter(catalog(ss),percentd=percentd,count=count,median_delta=median_delta,radiation_belt=radiation_belt)
;     --------- filter out bad frames ------------
      if count gt 0 then ss=ss(ssgood)                   ; filter out bad
      nrejects=thismovie.nmatches-count
      if count ne thismovie.nmatches then begin
         box_message,['Data Quality Filter', $
		      'Rejected ' + strtrim(nrejects,2) + ' images']
      endif	
      logstruct(uss(m)).NSS=n_elements(ss)
      logstruct(uss(m)).dqreject=nrejects
      case 1 of 
         count lt minframes: $
	       box_message,'Good frames dropped below MINFRAMES, aborting this movie'
         max(gt_tagval(catalog(ss),/naxis1)) lt minnx: $
            box_message, 'Rejecting "too small" images, aborting this movie'
         else: begin 

;     ------------- GOOD MOVIE BLOCK -------------------------
      case 1 of 
         count gt maxframes and keyword_set(grid):  begin
           box_message,['#Frames > MAXFRAMES ('+strtrim(maxframes,2), $
                        'Gridding MAXFRAMES over time range']
           ss=ss(grid_data(catalog(ss),nsamp=maxframes))
         endcase
         count gt maxframes: begin
            box_message, ['#Frames> MAXFRAMES ('+strtrim(maxframes,2) +')', $
                         'Taking most recent MAXFRAMES']
            ss=last_nelem(ss,maxframes)
         endcase
         else:
      endcase
  
      logstruct(uss(m)).catalog=catalog(ss)  
      
      if not testing then begin 
;     --------- map catalog->data --------------------
      trace_cat2data,catalog(ss),files,dset,/filedset       ; catalog->filedset
      read_trace, files, dset, index, data                  ; read data
      nx=gt_tagval(index(0),/naxis1,missing=0)
      wl=gt_tagval(index(0),/wave_len,missing='??')

;     ------- correct data (trace_prep) or just apply some scale ---------
      refimg=n_elements(index)/2               ; scale/rotation/... reference 
      refimg=0
      if prepit then begin
         box_message,'Prepping data...'
         trace_prep,index,data,oindex, odata, normalize=normalize, /unspike

         box_message,'Derotating data...'
         oindex=trace_sub2point(oindex)
         ssw_track_fov, oindex, odata, index, data, ref_image=refimg
         if is_member(wl(0),'171,195,284') and keyword_set(cube_unspike) then $
            data=ssw_unspike_cube(index, data, oindex)

;         if is_member(wl(0),'171,195,284') then begin
;            box_message,'Depiking data...'
;            for j=0,n_elements(index)-1 do data(*,*,j)= $
;               round(trace_unspike(data(*,*,j)))
;         endif  
      endif
      data=trace_scale(index, data, reference=refimg ,/byte, /force)

      ; filter data where MIN=MAX (indicates bad scaling)
      nimg=data_chk(data,/nimage)
      ss=lonarr(nimg)

      box_message,'Filtering data ....
      for qi=0, nimg-1 do ss(qi)=min(data(*,*,qi)) lt max(data(*,*,qi))
      ok=where(ss,sscnt)
      if sscnt lt nimg then begin
	 box_message,['Throwing out ' + strtrim(nimg-sscnt,2) + ' images...', $
		     strmid(trace_list_index(index(where(1-ss))),0,70)] 
         if sscnt eq 0 then begin 
            box_message,nbox=2,'WARNING - NO GOOD IMAGES...
         endif else begin 
            data=cube_edit(data,ss=ss(ok),/data)
	    index=index(ss(ok))
         endelse
      endif else box_message,'No images filtered out due to min=max'
      if sscnt gt 0 then begin 
      
;     if /CONTEXT set, include EIT/GOES context image via ssw_fov_context.pro
      cimg=0 & ctitle=''
      if keyword_set(context) then begin
         box_message,'Generating context images...'
         case 1 of
            n_elements(context) gt 1: csizes=[max(context),min(context)]
            context(0) gt 1: csizes=[context(0),140]
            else: csizes=[512,140]               ; default NX [primary,thumb] 
         endcase
         fov=[15,25]                             ; context FOV
         cnames=thismovie(0).uname+'_context_'+strtrim(csizes,2)+'.gif'
         ct2rgb,1,rr,gg,bb,gamma=.8
         for ci=0,n_elements(csizes)-1 do begin 
	    cimg=ssw_fov_context(index, data,/zbuffer,xsize=csizes(ci),/goes,rr,gg,bb,$
		 oplotnar=(ci eq 0), /midpoint, /rotate, offset=[100,100],$
                 fov=fov(ci),fdroll=fdroll, fd_type=fd_type)
            write_gif,concat_dir(mxdir,cnames(ci)),byte(cimg),rr,gg,bb
	 endfor
       endif  

      trace_colors,fix(thismovie.wave_len),r,g,b   ; wavelength->RGB

;     ---- data->WWW in 3 standard formats via special_movie (image2movie)
       
       if keyword_set(goes_movie) then begin 
          odata=cube_edit(data,nx=origsize(0),ny=origsize(0),/data)
          event_movie,index,data=odata,/goes,/reverse, data
          outsize=[data_chk(data,/nx), data_chk(data,/ny)]
       endif

       if not data_chk(fd_type,/string) then fdcont='EIT 171' else $
           fdcont=strupcase(fd_type)
       ctitle=fdcont + ' & GOES Context'
       special_movie, index, data, r, g, b, /no_html_doc, html=html, $
          movie_name=([mname+'_'+strtrim(i,2)+strtrim(m,2), $
                       thismovie(0).uname])(1),             $
          _extra=_extra, movie_dir=mxdir, /label, outsize=outsize, $
          context=cnames, ctitle=ctitle
          anchor='<a name="'+thismovie(0).uname+'"></a>'
          allhtml=[allhtml,anchor,"<h4>Wavelength: " + thismovie.wave_len + "&#197"+ $
		   "; Source Area: " +  strtrim(thismovie.sou_area,2)+"</h4>",html]
    endif   ; ---------- end FILTER OK    ----------------
    endif   ; ---------- end TESTING skip ------------------
    endcase ; ---------- end GOOD movie block --------------
    endcase ; ---------- end data filter case block --------
    endfor  ; ---------- end of submovie loop --------------

;   ================ write the top level html documnet ==============
    hdoc=concat_dir(mxdir,(['TSP_','LATEST_TRACE_'])(recent)+thisobsprog+'.html')
    clean_html=str_replace(allhtml,'http://./','')
    html_doc,hdoc,/header
    file_append, hdoc,'<br><em> Created by trace_special_movie.pro at ' + systime() + $
		 ', Last Revision 4-Jun-1999, S.L.Freeland</em><p>
    file_append, hdoc, clean_html
    html_doc,hdoc,/trailer
    tophtml=concat_dir(mxdir,top_name)

    if not recent then begin
       box_message,'Updating master html> ' + tophtml  ; update master html
       html_doc,   tophtml,/header
       file_append,tophtml,allhtml
       html_doc,   tophtml,/trailer
    endif
    
;   =====================================================

; ----------- (re)generate the top level html index file --------
endfor     ; ------- end of OBS_PROG loop -------------------

; -------- Write the GENX log ----------------
fmt_timer,catalog,t0,t1
savegen,file=genlog,t0, t1,logstruct, names='time0,time1,uminfo'
box_message, 'Wrote GENX log file >> ' + genlog
if not testing then begin
   trace_movie_index,mdir=mxdir
endif
; -----------------------------------------------
; if recent, move current->lm_YYMMDDD
if keyword_set(last_nhours) then begin
   box_message,'Updating recent movie queue'
   trace_last_movie_queue, queue_days            ; handle movie queue directories
   trace_movie_index,/anchor                     ; make index.html
   trace_movie_qkl_queue,/insert                 ; add queue links->index
endif

; =====================================================
return
end
